@@ -10,6 +10,10 @@ use PPIx::Regexp::Constant qw{
     ARRAY_REF
     CODE_REF
     HASH_REF
+    LOCATION_LINE
+    LOCATION_CHARACTER
+    LOCATION_COLUMN
+    LOCATION_LOGICAL_LINE
     MINIMUM_PERL
     REGEXP_REF
     TOKEN_LITERAL
@@ -46,10 +50,14 @@ use PPIx::Regexp::Token::Recursion		();
 use PPIx::Regexp::Token::Structure		();
 use PPIx::Regexp::Token::Unknown		();
 use PPIx::Regexp::Token::Whitespace		();
-use PPIx::Regexp::Util qw{ is_ppi_regexp_element __instance };
+use PPIx::Regexp::Util qw{
+    is_ppi_regexp_element
+    __instance
+};
+
 use Scalar::Util qw{ looks_like_number };
 
-our $VERSION = '0.069';
+our $VERSION = '0.085';
 
 our $DEFAULT_POSTDEREF;
 defined $DEFAULT_POSTDEREF
@@ -123,6 +131,7 @@ defined $DEFAULT_POSTDEREF
 	    };
 
 	my $self = {
+	    index_locations => $args{index_locations},	# Index locations
 	    capture => undef,	# Captures from find_regexp.
 	    content => undef,	# The string we are tokenizing.
 	    cookie => {},	# Cookies
@@ -142,13 +151,11 @@ defined $DEFAULT_POSTDEREF
 	    failures => 0,	# Number of parse failures.
 	    find => undef,	# String for find_regexp
 	    known => {},	# Known tokenizers, by mode.
+	    location => $args{location},
 	    match => undef,	# Match from find_regexp.
 	    mode => 'init',	# Initialize
 	    modifiers => [{}],	# Modifier hash.
 	    pending => [],	# Tokens made but not returned.
-	    postderef => defined $args{postderef} ?
-		$args{postderef} :
-		$DEFAULT_POSTDEREF,
 	    prior => TOKEN_UNKNOWN,	# Prior significant token.
 	    source => $re,	# The object we were initialized with.
 	    strict => $args{strict},	# like "use re 'strict';".
@@ -211,6 +218,10 @@ sub cookie {
     my ( $self, $name, @args ) = @_;
     defined $name
 	or confess "Programming error - undefined cookie name";
+    if ( $self->{trace} ) {
+	local $" = ', ';
+	warn "cookie( '$name', @args )\n";
+    }
     @args or return $self->{cookie}{$name};
     my $cookie = shift @args;
     if ( CODE_REF eq ref $cookie ) {
@@ -222,6 +233,20 @@ sub cookie {
     }
 }
 
+# NOTE: Currently this is called only against
+# COOKIE_LOOKAROUND_ASSERTION, once in PPIx::Token::GroupType::Assertion
+# to prevent the cookie from being remade if it already exists, and once
+# in PPIx::Regexp::Token::Assertion to determine if \K is inside a
+# lookaround assertion. If it gets used other places, or if there is
+# call for it, I should consider removing the underscores and
+# documenting it as public.
+sub __cookie_exists {
+    my ( $self, $name ) = @_;
+    defined $name
+	or confess "Programming error - undefined cookie name";
+    return $self->{cookie}{$name};
+}
+
 sub default_modifiers {
     my ( $self ) = @_;
     return [ @{ $self->{default_modifiers} } ];
@@ -413,6 +438,9 @@ sub make_token {
 	%{ $arg || {} } )
 	or return;
 
+    $self->{index_locations}
+	and $self->_update_location( $token );
+
     $token->significant()
 	and $self->{expect} = undef;
 
@@ -546,10 +574,7 @@ sub prior_significant_token {
 # This method is private to the PPIx-Regexp package, and may be changed
 # or retracted without warning. What it does is to recognize postfix
 # dereferences. It returns the length in characters of the first postfix
-# dereference found, or a false value if none is found. This returns
-# false immediately unless the tokenizer was instantiated with the
-# C<postderef> argument true, or if it was not specified and
-# C<$DEFAULT_POSTDEREF> was true when the tokenizer was instantiated.
+# dereference found, or a false value if none is found.
 #
 # The optional $iterator argument can be one of the following:
 #   - A code reference, which will be called to provide PPI::Element
@@ -563,20 +588,18 @@ sub prior_significant_token {
 #   - Anything else results in an exception and stack trace.
 
 {
-    # %* &* **
-    my %magic_var = map { $_ => 1 } qw{ @* $* };
-    my %magic_oper = map { $_ => 1 } qw{ & ** % };
-    my %sliceable = map { $_ => 1 } qw{ @ % };
-    my %post_slice = map { $_ => 1 } qw< { [ >;	# ] }
-
     sub __recognize_postderef {
 	my ( $self, $token, $iterator ) = @_;
-	$self->{postderef}
-	    or return;
+
 	# Note that if ppi() gets called I have to hold a reference to
 	# the returned object until I am done with all its children.
 	my $ppi;
 	if ( ! defined $iterator ) {
+
+	    # This MUST be done before ppi() is called.
+	    $self->{index_locations}
+		and $self->_update_location( $token );
+
 	    $ppi = $token->ppi();
 	    my @ops = grep { '->' eq $_->content() } @{
 		$ppi->find( 'PPI::Token::Operator' ) || [] };
@@ -599,52 +622,26 @@ sub prior_significant_token {
 	while ( my $elem = $iterator->() ) {
 
 	    my $content = $elem->content();
-	    $content =~ m/ \A ( . \#? ) /smx
-		and $accept->{$1}
+
+	    # As of PPI 1.238, all postfix dereferences are parsed as
+	    # casts. So if we find a cast of the correct content we have
+	    # a postfix deref.
+	    $elem->isa( 'PPI::Token::Cast' )
 		or next;
 
-	    my $length = length $content;
-
-	    # PPI parses '$x->@*' as containing magic variable '@*'.
-	    # Similarly for '$*' and '$#*'. I think this is a bug, and
-	    # they should be parsed as casts, but ...
-	    if ( $elem->isa( 'PPI::Token::Magic' ) ) {
-		$magic_var{$content}
-		    and return $length;
-		if ( '$#' eq $content ) {
-		    my $next = $elem->snext_sibling()
-			or return $length;
-		    '*' eq substr $next->content(), 0, 1
-			and return $length + 1;
-		}
-	    }
-
-	    # PPI parses '%*' as a cast of '%' followed by a splat, but
-	    # I think it is likely that if it ever supports postderef
-	    # operators that they will be casts. It currently parses
-	    # '**' as an operator and '&*' as two operators, but the
-	    # logic is pretty much the same as for a cast, so they get
-	    # handled here too.
-	    if ( $elem->isa( 'PPI::Token::Cast' ) || $elem->isa(
-		    'PPI::Token::Operator' ) && $magic_oper{$content} ) {
-		# Maybe PPI will eventually parse something like '$*' as
-		# a cast, so ...
-		$content =~ m/ [*] \z /smx
-		    and return $length;
-		# Or maybe it will parse the asterisk separately, but I
-		# have no idea what its class will be.
+	    if ( $content =~ m/ ( .* ) \* \z /smx ) {
+		# If we're an acceptable cast ending in a glob, accept
+		# it.
+		$accept->{$1}
+		    and return length $content;
+	    } elsif ( $accept->{$content} ) {
+		# If we're an acceptable cast followed by a subscript,
+		# we're a slice -- accept both cast and subscript.
 		my $next = $elem->snext_sibling()
-		    or return;
-		my $next_content = $next->content();
-		my $next_char = substr $next_content, 0, 1;
-		'*' eq $next_char
-		    and return $length + 1;
-		# We may still have a slice.
-		$sliceable{$content}
-		    and $post_slice{$next_char}
-		    and return $length + length $next_content;
-		# TODO maybe PPI will do something completely
-		# unanticipated with postderef.
+		    or next;
+		$next->isa( 'PPI::Structure::Subscript' )
+		    or next;
+		return length( $content ) + length( $next->content() );
 	    }
 
 	    # Otherwise, we're not a postfix dereference; try the next
@@ -738,9 +735,7 @@ sub tokens {
 
     my %deprecate = (
 	attribute => {
-	},
-	method => {
-	    prior	=> 3,
+	    postderef	=> 3,
 	},
     );
 
@@ -817,6 +812,63 @@ sub __init_error {
     );
 }
 
+sub _update_location {
+    my ( $self, $token ) = @_;
+    $token->{location}	# Idempotent
+	and return;
+    my $loc = $self->{_location} ||= do {
+	my %loc = (
+	    location	=> $self->{location},
+	);
+	if ( __instance( $self->{source}, 'PPI::Element' ) ) {
+	    $loc{location} ||= $self->{source}->location();
+	    if ( my $doc = $self->{source}->document() ) {
+		$loc{tab_width} = $doc->tab_width();
+	    }
+	}
+	$loc{tab_width} ||= 1;
+	\%loc;
+    };
+    $loc->{location}
+	or return;
+    $token->{location} = [ @{ $loc->{location} } ];
+    if ( defined( my $content = $token->content() ) ) {
+
+	my $lines;
+	pos( $content ) = 0;
+	$lines++ while $content =~ m/ \n /smxgc;
+	if ( pos $content ) {
+	    $loc->{location}[LOCATION_LINE] += $lines;
+	    $loc->{location}[LOCATION_LOGICAL_LINE] += $lines;
+	    $loc->{location}[LOCATION_CHARACTER] =
+		$loc->{location}[LOCATION_COLUMN] = 1;
+	}
+
+	if ( my $chars = length( $content ) - pos( $content ) ) {
+	    $loc->{location}[LOCATION_CHARACTER] += $chars;
+	    if ( $loc->{tab_width} > 1 && $content =~ m/ \t /smx ) {
+		my $pos = $loc->{location}[LOCATION_COLUMN];
+		my $tab_width = $loc->{tab_width};
+		# Stolen shamelessly from PPI::Document::_visual_length
+		my ( $vis_inc );
+		foreach my $part ( split /(\t)/, $content ) {
+		    if ($part eq "\t") {
+			$vis_inc = $tab_width - ($pos-1) % $tab_width;
+		    } else {
+			$vis_inc = length $part;
+		    }
+		    $pos    += $vis_inc;
+		}
+		$loc->{location}[LOCATION_COLUMN] = $pos;
+	    } else {
+		$loc->{location}[LOCATION_COLUMN] += $chars;
+	    }
+	}
+
+    }
+    return;
+}
+
 sub __PPIX_TOKENIZER__init {
     my ( $self ) = @_;
 
@@ -929,6 +981,13 @@ sub __PPIX_TOKENIZER__init {
     }
 
     {
+	# We have to instantiate the trailing tokens now so we can
+	# figure out what modifiers are in effect. But we can't
+	# index their locations (if desired) because they are being
+	# instantiated out of order
+
+	local $self->{index_locations} = 0;
+
 	my @mods = @{ $self->{default_modifiers} };
 	pos $self->{content} = $self->{cursor_modifiers};
 	local $self->{cursor_curr} = $self->{cursor_modifiers};
@@ -1105,8 +1164,8 @@ sub __PPIX_TOKENIZER__finish {
 
 	# We are out of string. Add the trailing tokens (created when we
 	# did the initial bracket scan) and close up shop.
-
-	push @tokens, @{ delete $self->{trailing_tokens} };
+	
+	push @tokens, $self->_get_trailing_tokens();
 
 	$self->_set_mode( 'kaput' );
 
@@ -1164,7 +1223,7 @@ sub __PPIX_TOKENIZER__finish {
 	    $self->{cursor_limit} = length $self->{content};
 	    push @tokens, $self->make_token( 1,
 		'PPIx::Regexp::Token::Delimiter' ),
-		@{ delete $self->{trailing_tokens} };
+		$self->_get_trailing_tokens();
 	    $self->_set_mode( 'kaput' );
 	} else {
 	    # Put our mode to replacement.
@@ -1177,6 +1236,19 @@ sub __PPIX_TOKENIZER__finish {
 
 }
 
+# To common processing on trailing tokens.
+sub _get_trailing_tokens {
+    my ( $self ) = @_;
+    if ( $self->{index_locations} ) {
+	# We turned off index_locations when these were created, because
+	# they were done out of order. Fix that now.
+	foreach my $token ( @{ $self->{trailing_tokens} } ) {
+	    $self->_update_location( $token );
+	}
+    }
+    return @{ delete $self->{trailing_tokens} };
+}
+
 1;
 
 __END__
@@ -1234,13 +1306,10 @@ This option specifies the encoding of the string to be tokenized. If
 specified, an C<Encode::decode> is done on the string (or the C<content>
 of the PPI class) before it is tokenized.
 
-=item postderef boolean
+=item index_locations
 
-This option specifies whether the tokenizer recognizes postfix
-dereferencing. See the L<PPIx::Regexp|PPIx::Regexp>
-L<new()|PPIx::Regexp/new> documentation for the details.
-
-C<$PPIx::Regexp::Tokenizer::DEFAULT_POSTDEREF> is not exported.
+This Boolean option specifies that the locations of the generated tokens
+are to be computed.
 
 =item strict boolean
 
@@ -1623,7 +1692,9 @@ troubleshoot funny output from the tokenizer.
 =head1 SUPPORT
 
 Support is by the author. Please file bug reports at
-L<https://rt.cpan.org>, or in electronic mail to the author.
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=PPIx-Regexp>,
+L<https://github.com/trwyant/perl-PPIx-Regexp/issues>, or in
+electronic mail to the author.
 
 =head1 AUTHOR
 
@@ -1631,7 +1702,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2020 by Thomas R. Wyant, III
+Copyright (C) 2009-2022 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
